/**
 * @constructor
 * @extends {ol.interaction.Pointer}
 */
ol.interaction.DragFeature = function(data) {

  var options = data.opt;

  ol.interaction.Pointer.call(this, {
    handleDownEvent: ol.interaction.DragFeature.prototype.handleDownEvent,
    handleDragEvent: ol.interaction.DragFeature.prototype.handleDragEvent,
    handleMoveEvent: ol.interaction.DragFeature.prototype.handleMoveEvent,
    handleUpEvent: ol.interaction.DragFeature.prototype.handleUpEvent
  });

  /**
   * @type {ol.Pixel}
   * @private
   */
  this.coordinate_ = null;

  /**
   * @type {string|undefined}
   * @private
   */
  this.cursor_ = 'pointer';

  /**
   * @type {ol.Feature}
   * @private
   */
  this.feature_ = null;

  /**
   * @type {string|undefined}
   * @private
   */
  this.previousCursor_ = undefined;

  if (typeof data.objects.styles[options.style] !== 'undefined') {
    this.dragFeatureStyle_ = data.objects.styles[options.style];
  } else {
    this.dragFeatureStyle_ = ol.style.defaultStyleFunction;
  }

  this.style_ = null;
};
ol.inherits(ol.interaction.DragFeature, ol.interaction.Pointer);


/**
 * @param {ol.MapBrowserEvent} evt Map browser event.
 * @return {boolean} `true` to start the drag sequence.
 */
ol.interaction.DragFeature.prototype.handleDownEvent = function(evt) {
  var map = evt.map;

  var feature = map.forEachFeatureAtPixel(evt.pixel,
    function(feature, layer) {
      return feature;
    });

  if (feature) {
    this.coordinate_ = evt.coordinate;
    this.feature_ = feature;
    this.style_ = feature.getStyleFunction();
  }

  return !!feature;
};


/**
 * @param {ol.MapBrowserEvent} evt Map browser event.
 */
ol.interaction.DragFeature.prototype.handleDragEvent = function(evt) {
  var map = evt.map;

  var feature = map.forEachFeatureAtPixel(evt.pixel,
    function(feature, layer) {
      return feature;
    });

  var deltaX = evt.coordinate[0] - this.coordinate_[0];
  var deltaY = evt.coordinate[1] - this.coordinate_[1];

  var geometry = /** @type {ol.geom.SimpleGeometry} */
    (this.feature_.getGeometry());
  geometry.translate(deltaX, deltaY);

  this.feature_.setStyle(this.dragFeatureStyle_(feature));

  this.coordinate_[0] = evt.coordinate[0];
  this.coordinate_[1] = evt.coordinate[1];
};


/**
 * @param {ol.MapBrowserEvent} evt Event.
 */
ol.interaction.DragFeature.prototype.handleMoveEvent = function(evt) {
  if (this.cursor_) {
    var map = evt.map;
    var feature = map.forEachFeatureAtPixel(evt.pixel,
      function(feature, layer) {
        return feature;
      });
    var element = evt.map.getTargetElement();
    if (feature) {
      if (element.style.cursor != this.cursor_) {
        this.previousCursor_ = element.style.cursor;
        element.style.cursor = this.cursor_;
      }
    } else if (this.previousCursor_ !== undefined) {
      element.style.cursor = this.previousCursor_;
      this.previousCursor_ = undefined;
    }
  }
};


/**
 * @param {ol.MapBrowserEvent} evt Map browser event.
 * @return {boolean} `false` to stop the drag sequence.
 */
ol.interaction.DragFeature.prototype.handleUpEvent = function(evt) {
  this.coordinate_ = null;
  if (typeof this.style_ === 'undefined') {
    this.feature_.setStyle(ol.style.defaultStyleFunction);
  } else {
    this.feature_.setStyle(this.style_(this.feature_));
  }
  this.style_ = null;
  this.feature_ = null;
  return false;
};
